@@ -47,6 +47,64 @@ glade_popup_select_cb (GtkMenuItem *item, GladeWidget *widget)
 		(glade_widget_get_object (widget), TRUE);
 }
 
+static GladePlaceholder *
+find_placeholder (GObject *object)
+{
+	GtkContainer *container;
+	GladePlaceholder *retval = NULL;
+	GtkWidget *child;
+	GList *c, *l;
+
+	if (!GTK_IS_CONTAINER (object))
+		return NULL;
+
+	container = GTK_CONTAINER (object);
+	
+	for (c = l = glade_util_container_get_all_children (container);
+	     l;
+	     l = g_list_next (l))
+	{
+		child = l->data;
+		
+		if (GLADE_IS_PLACEHOLDER (child))
+		{
+			retval = GLADE_PLACEHOLDER (child);
+			break;
+		}
+	}
+	
+	g_list_free (c);
+
+	return retval;
+}
+
+static void
+glade_popup_placeholder_add_cb (GtkMenuItem *item, GladePlaceholder *placeholder)
+{
+	GladeWidgetAdaptor *adaptor;
+	
+	adaptor = glade_palette_get_current_item (glade_app_get_palette ());
+	g_return_if_fail (adaptor != NULL);
+	
+	glade_command_create (adaptor, glade_placeholder_get_parent (placeholder),
+						  placeholder, glade_placeholder_get_project (placeholder));
+						  
+	glade_palette_deselect_current_item (glade_app_get_palette(), TRUE);
+}
+
+static void
+glade_popup_root_add_cb (GtkMenuItem *item, gpointer *user_data)
+{
+	GladeWidgetAdaptor *adaptor = (GladeWidgetAdaptor *)user_data;
+	GladePalette *palette = glade_app_get_palette ();
+	
+	if (!adaptor)
+		adaptor = glade_palette_get_current_item (palette);
+	g_return_if_fail (adaptor != NULL);
+	
+	glade_palette_create_root_widget (palette, adaptor);
+}
+
 static void
 glade_popup_cut_cb (GtkMenuItem *item, GladeWidget *widget)
 {
@@ -86,7 +144,10 @@ glade_popup_paste_cb (GtkMenuItem *item, GladeWidget *widget)
 	glade_util_clear_selection ();
 
 	/* The selected widget is the paste destination */
-	glade_app_selection_set	(glade_widget_get_object (widget), FALSE);
+	if (widget)
+		glade_app_selection_set	(glade_widget_get_object (widget), FALSE);
+	else
+		glade_app_selection_clear (FALSE);
 
 	glade_app_command_paste (NULL);
 }
@@ -138,6 +199,20 @@ glade_popup_clipboard_delete_cb (GtkMenuItem *item, GladeWidget *widget)
 	glade_app_command_delete_clipboard ();
 }
 
+static void
+glade_popup_docs_cb (GtkMenuItem *item,
+		     GladeWidgetAdaptor *adaptor)
+{
+	gchar *book;
+
+	g_return_if_fail (GLADE_IS_WIDGET_ADAPTOR (adaptor));
+
+	g_object_get (adaptor, "book", &book, NULL);
+	glade_editor_search_doc_search (glade_app_get_editor (), book, adaptor->name, NULL);
+	g_free (book);
+}
+
+
 /********************************************************
                     POPUP BUILDING
  *******************************************************/
@@ -145,6 +220,7 @@ static GtkWidget *
 glade_popup_append_item (GtkWidget *popup_menu,
 			 const gchar *stock_id,
 			 const gchar *label,
+			 GtkWidget   *image,
 			 gboolean sensitive,
 			 gpointer callback,
 			 gpointer data)
@@ -157,6 +233,11 @@ glade_popup_append_item (GtkWidget *popup_menu,
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
 			gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU));
 	}
+	else if (image && label)
+	{
+		menu_item = gtk_image_menu_item_new_with_mnemonic (label);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);
+	}
 	else if (stock_id)
 		menu_item = gtk_image_menu_item_new_from_stock (stock_id, NULL);
 	else
@@ -215,6 +296,7 @@ glade_popup_menuitem_ph_packing_activated (GtkMenuItem *item, const gchar *actio
 
 static gint
 glade_popup_action_populate_menu_real (GtkWidget *menu,
+				       GladeWidget *gwidget,
 				       GList *actions,
 				       GCallback callback,
 				       gpointer data)
@@ -232,14 +314,20 @@ glade_popup_action_populate_menu_real (GtkWidget *menu,
 		{
 			submenu = gtk_menu_new ();
 			n += glade_popup_action_populate_menu_real (submenu,
+								    gwidget,
 								    a->actions,
 								    callback,
 								    data);
 		}
+		else
+			submenu = glade_widget_adaptor_action_submenu (gwidget->adaptor,
+								       gwidget->object,
+								       a->klass->path);
+				
 				
 		item = glade_popup_append_item (menu, 
 						a->klass->stock,
-						a->klass->label, TRUE,
+						a->klass->label, NULL, TRUE,
 						(a->actions) ? NULL : callback,
 						(a->actions) ? NULL : a->klass->path);
 		
@@ -283,12 +371,14 @@ glade_popup_action_populate_menu (GtkWidget *menu,
 		g_return_val_if_fail (GLADE_IS_WIDGET_ACTION (action), 0);
 		if (glade_widget_get_action (widget, action->klass->path))
 			return glade_popup_action_populate_menu_real (menu,
+								      widget,
 								      action->actions,
 								      G_CALLBACK (glade_popup_menuitem_activated),
 								      widget);
 		
 		if (glade_widget_get_pack_action (widget, action->klass->path))
 			return glade_popup_action_populate_menu_real (menu,
+								      glade_widget_get_parent (widget),
 								      action->actions,
 								      G_CALLBACK (glade_popup_menuitem_packing_activated),
 								      widget);
@@ -297,6 +387,7 @@ glade_popup_action_populate_menu (GtkWidget *menu,
 	}
 	
 	n = glade_popup_action_populate_menu_real (menu,
+						   widget,
 						   widget->actions,
 						   G_CALLBACK (glade_popup_menuitem_activated),
 						   widget);
@@ -310,6 +401,7 @@ glade_popup_action_populate_menu (GtkWidget *menu,
 			gtk_widget_show (separator);
 		}
 		n += glade_popup_action_populate_menu_real (menu,
+							    glade_widget_get_parent (widget),
 							    widget->packing_actions,
 							    G_CALLBACK (glade_popup_menuitem_packing_activated),
 							    widget);
@@ -323,35 +415,79 @@ glade_popup_create_menu (GladeWidget      *widget,
 			 GladePlaceholder *placeholder,
 			 gboolean          packing)
 {
+	GladeProjectFormat fmt;
+	GladeProject *project;
 	GtkWidget *popup_menu;
-	gboolean   sensitive;
+	GtkWidget *separator;
+	GList *list;
+	gboolean   sensitive, non_window;
+	GladePlaceholder *tmp_placeholder;
+	gchar     *book;
+
+	sensitive = glade_palette_get_current_item (glade_app_get_palette ()) != NULL;
+
+	/* Resolve project format first... */
+	project = widget ? glade_widget_get_project (widget) :
+		placeholder ? glade_placeholder_get_project (placeholder) : glade_app_get_project ();
+	fmt = glade_project_get_format (project);
+
 
 	popup_menu = gtk_menu_new ();
+	
+	if (sensitive)
+	{		
+		tmp_placeholder = placeholder;
+		if (!tmp_placeholder && widget)
+			tmp_placeholder = find_placeholder (glade_widget_get_object (widget));
+		glade_popup_append_item (popup_menu, NULL, _("_Add widget here"), NULL, tmp_placeholder != NULL,
+					 glade_popup_placeholder_add_cb, tmp_placeholder);
+		glade_popup_append_item (popup_menu, NULL, _("Add widget as _toplevel"), NULL, 
+					 fmt != GLADE_PROJECT_FORMAT_LIBGLADE,
+					 glade_popup_root_add_cb, NULL);
 
-	glade_popup_append_item (popup_menu, NULL, _("_Select"), TRUE,
+		separator = gtk_menu_item_new ();		
+		gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), separator);
+		gtk_widget_show (separator);
+	}
+
+	sensitive = (widget != NULL);
+
+	glade_popup_append_item (popup_menu, NULL, _("_Select"), NULL, sensitive,
 				 glade_popup_select_cb, widget);
-	glade_popup_append_item (popup_menu, GTK_STOCK_CUT, NULL, TRUE,
+	glade_popup_append_item (popup_menu, GTK_STOCK_CUT, NULL, NULL, sensitive,
 				 glade_popup_cut_cb, widget);
-	glade_popup_append_item (popup_menu, GTK_STOCK_COPY, NULL, TRUE,
+	glade_popup_append_item (popup_menu, GTK_STOCK_COPY, NULL, NULL, sensitive,
 				 glade_popup_copy_cb, widget);
 
 	/* paste is placholder specific when the popup is on a placeholder */
 	sensitive = glade_clipboard_get_has_selection (glade_app_get_clipboard ());
-	if (placeholder)
-		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, sensitive,
-					 glade_popup_placeholder_paste_cb, placeholder);
-	else
-		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, sensitive,
-					 glade_popup_paste_cb, widget);
+	non_window = FALSE;
+	for (list = glade_app_get_clipboard ()->selection; list; list = list->next)
+		if (!GTK_IS_WINDOW (GLADE_WIDGET (list->data)->object))
+			non_window = TRUE;
 
-	glade_popup_append_item (popup_menu, GTK_STOCK_DELETE, NULL, TRUE,
+	if (placeholder)
+		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, NULL, sensitive,
+					 glade_popup_placeholder_paste_cb, placeholder);
+	else if (widget)
+		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, NULL, sensitive,
+					 glade_popup_paste_cb, widget);
+	else
+		/* No toplevel non-GtkWindow pastes in libglade */
+		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, NULL, 
+					 sensitive && !(non_window &&  (fmt == GLADE_PROJECT_FORMAT_LIBGLADE)),
+					 glade_popup_paste_cb, NULL);
+
+
+
+	glade_popup_append_item (popup_menu, GTK_STOCK_DELETE, NULL, NULL, (widget != NULL),
 				 glade_popup_delete_cb, widget);
 
 
 	/* packing actions are a little different on placholders */
 	if (placeholder)
 	{
-		if (widget->actions)
+		if (widget && widget->actions)
 		{
 			GtkWidget *separator = gtk_menu_item_new ();
 			gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), separator);
@@ -359,6 +495,7 @@ glade_popup_create_menu (GladeWidget      *widget,
 
 			glade_popup_action_populate_menu_real
 				(popup_menu,
+				 widget,
 				 widget->actions,
 				 G_CALLBACK (glade_popup_menuitem_activated),
 				 widget);
@@ -372,12 +509,13 @@ glade_popup_create_menu (GladeWidget      *widget,
 			
 			glade_popup_action_populate_menu_real
 				(popup_menu,
+				 widget,
 				 placeholder->packing_actions,
 				 G_CALLBACK (glade_popup_menuitem_ph_packing_activated),
 				 placeholder);
 		}
 	}
-	else if (widget->actions || (packing && widget->packing_actions))
+	else if (widget && (widget->actions || (packing && widget->packing_actions)))
 	{
 		GtkWidget *separator = gtk_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), separator);
@@ -386,6 +524,22 @@ glade_popup_create_menu (GladeWidget      *widget,
 		glade_popup_action_populate_menu (popup_menu, widget, NULL, packing);
 	}
 
+	if (widget)
+	{
+		g_object_get (widget->adaptor, "book", &book, NULL);
+		if (book)
+		{
+			GtkWidget *icon = glade_util_get_devhelp_icon (GTK_ICON_SIZE_MENU);
+			GtkWidget *separator = gtk_menu_item_new ();
+			gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), separator);
+			gtk_widget_show (separator);
+			
+			glade_popup_append_item (popup_menu, NULL, _("Read _documentation"), icon, TRUE,
+						 glade_popup_docs_cb, widget->adaptor);
+		}
+		g_free (book);
+	}
+
 	return popup_menu;
 }
 
@@ -399,11 +553,11 @@ glade_popup_create_clipboard_menu (GladeWidget *widget)
 	if (GTK_WIDGET_TOPLEVEL (glade_widget_get_object (widget)))
 	{
 		glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, 
-					 NULL, TRUE,
+					 NULL, NULL, TRUE,
 					 glade_popup_clipboard_paste_cb, NULL);
 	}
 
-	glade_popup_append_item (popup_menu, GTK_STOCK_DELETE, NULL, TRUE,
+	glade_popup_append_item (popup_menu, GTK_STOCK_DELETE, NULL, NULL, TRUE,
 				 glade_popup_clipboard_delete_cb, widget);
 
 	return popup_menu;
@@ -418,7 +572,7 @@ glade_popup_widget_pop (GladeWidget *widget,
 	gint button;
 	gint event_time;
 
-	g_return_if_fail (GLADE_IS_WIDGET (widget));
+	g_return_if_fail (GLADE_IS_WIDGET (widget) || widget == NULL);
 
 	popup_menu = glade_popup_create_menu (widget, NULL, packing);
 
@@ -492,3 +646,154 @@ glade_popup_clipboard_pop (GladeWidget    *widget,
 	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
 			NULL, NULL, button, event_time);
 }
+
+void
+glade_popup_palette_pop (GladeWidgetAdaptor *adaptor,
+			 GdkEventButton     *event)
+{
+	GladeProjectFormat fmt;
+	GladeProject *project;
+	GtkWidget *popup_menu;
+	gchar *book = NULL;
+	gint button;
+	gint event_time;
+
+	g_return_if_fail (GLADE_IS_WIDGET_ADAPTOR (adaptor));
+
+	popup_menu = gtk_menu_new ();
+	
+	project = glade_app_get_project ();
+	fmt = glade_project_get_format (project);
+
+	glade_popup_append_item (popup_menu, NULL, _("Add widget as _toplevel"), NULL, 
+				 (fmt != GLADE_PROJECT_FORMAT_LIBGLADE),
+				 glade_popup_root_add_cb, adaptor);
+
+	g_object_get (adaptor, "book", &book, NULL);
+	if (book && glade_util_have_devhelp ())
+	{
+		GtkWidget *icon = glade_util_get_devhelp_icon (GTK_ICON_SIZE_MENU);
+		glade_popup_append_item (popup_menu, NULL, _("Read _documentation"), icon, TRUE,
+					 glade_popup_docs_cb, adaptor);
+	}
+	g_free (book);
+
+
+	if (event)
+	{
+		button = event->button;
+		event_time = event->time;
+	}
+	else
+	{
+		button = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+
+	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
+			NULL, NULL, button, event_time);
+}
+
+static void
+glade_popup_clear_property_cb (GtkMenuItem   *item,
+			       GladeProperty *property)
+{
+	GValue value = { 0, };
+
+	glade_property_get_default (property, &value);
+	glade_command_set_property_value (property, &value);
+	g_value_unset (&value);
+}
+
+static void
+glade_popup_property_docs_cb (GtkMenuItem   *item,
+			      GladeProperty *property)
+{
+	GladeWidgetAdaptor *adaptor, *prop_adaptor;
+	gchar              *search, *book;
+
+	prop_adaptor = glade_widget_adaptor_from_pclass (property->klass);
+	adaptor = glade_widget_adaptor_from_pspec (prop_adaptor, property->klass->pspec);
+	search  = g_strdup_printf ("The %s property", property->klass->id);
+
+	g_object_get (adaptor, "book", &book, NULL);
+
+	glade_editor_search_doc_search (glade_app_get_editor (),
+					book, g_type_name (property->klass->pspec->owner_type), search);
+
+	g_free (book);
+	g_free (search);
+}
+
+void
+glade_popup_property_pop (GladeProperty  *property,
+			  GdkEventButton *event)
+{
+
+	GladeWidgetAdaptor *adaptor, *prop_adaptor;
+	GtkWidget *popup_menu;
+	gchar *book = NULL;
+	gint button;
+	gint event_time;
+
+	prop_adaptor = glade_widget_adaptor_from_pclass (property->klass);
+	adaptor = glade_widget_adaptor_from_pspec (prop_adaptor, property->klass->pspec);
+
+	g_return_if_fail (GLADE_IS_WIDGET_ADAPTOR (adaptor));
+
+	popup_menu = gtk_menu_new ();
+
+	glade_popup_append_item (popup_menu, GTK_STOCK_CLEAR, _("Set default value"), NULL, 
+				 TRUE, glade_popup_clear_property_cb, property);
+
+	g_object_get (adaptor, "book", &book, NULL);
+	if (book && glade_util_have_devhelp ())
+	{
+		GtkWidget *icon = glade_util_get_devhelp_icon (GTK_ICON_SIZE_MENU);
+		glade_popup_append_item (popup_menu, NULL, _("Read _documentation"), icon, TRUE,
+					 glade_popup_property_docs_cb, property);
+	}
+	g_free (book);
+
+
+	if (event)
+	{
+		button = event->button;
+		event_time = event->time;
+	}
+	else
+	{
+		button = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+
+	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
+			NULL, NULL, button, event_time);
+}
+
+
+void
+glade_popup_simple_pop (GdkEventButton *event)
+{
+	GtkWidget *popup_menu;
+	gint button;
+	gint event_time;
+	
+	popup_menu = glade_popup_create_menu (NULL, NULL, FALSE);
+	if (!popup_menu)
+		return;
+
+	if (event)
+	{
+		button = event->button;
+		event_time = event->time;
+	}
+	else
+	{
+		button = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
+		NULL, NULL, button, event_time);
+}
+
